#include <iostream>
#include <gtk--/main.h>
#include <gtk--/window.h>
#include <gtk--/button.h>
#include <gtk--/checkbutton.h>
#include <gtk--/box.h>
#include <gtk--/frame.h>
#include <gtk--/table.h>
#include <gtk--/scale.h>
#include <gtk--/adjustment.h>
#include <gtk--/label.h>
#include <gtk--/alignment.h>
#include <gtk--/menubar.h>
#include <gtk--/menu.h>
#include <gtk--/menuitem.h>
#include <gtk--/fileselection.h>

#include "ossmixer.hh"

using std::cout;
using SigC::slot;
using SigC::bind;

void destroy_handler()
{
  Gtk::Main::quit();
}

class MixerGUI: public Gtk::Window {
  ossMixer myMixer; // interface to mixer
  Gtk::VBox mainbox; // main box :)
  Gtk::Frame channelframe; // Frame to contain scaletable
  Gtk::Table *scaletable; // Table with all scaleboxes in...
  Gtk::VScale **scaleboxes;
  Gtk::Adjustment **adj; // List with the levels of all available devices...
  Gtk::CheckButton **mutebuttons;
  Gtk::Label *statusbar,*aboutlabel;
  Gtk::MenuBar *menubar;
  Gtk::MenuItem **menuitems;
  Gtk::Menu *filemenu,*helpmenu;
  Gtk::FileSelection *fsel;
  char *lastload,*lastsave,*title;
public:
  MixerGUI(char *);
  void quit();
  void callback(int);
  void mute(int);
  void SaveDefault(void);
  void LoadDefaults(void);
  void SaveCallback(void);
  void LoadCallback(void);
  void Save(void);
  void Load(void);
  gint UpdateLevels(void);
  void HelpWindow(void);
  void AboutWindow(void);
  gint TimerCallback(void);
};

gint MixerGUI::TimerCallback(void){
  statusbar->set(title);
  return true;
};

void MixerGUI::AboutWindow(void){
  Gtk::Window *window = manage(new Gtk::Window());
  window->set_modal(true);
  window->set_policy(0,0,0);
  window->set_usize(300,200);
  window->set_border_width(5);
  window->set_title("about opMixer");
  Gtk::Frame *aboutframe=manage(new Gtk::Frame("about opMixer"));
  char *info=new char[1024];
  sprintf(info,"%s\n\n by\n\n wizborn <igge_rask@yahoo.com>\n with a little help from\n comlord <comlord@yahoo.com>",title);
  aboutframe->set_label_align(0.9);
  aboutframe->add(*manage(new Gtk::Label(info)));
  window->add(*aboutframe);
  window->show_all();
};

void MixerGUI::HelpWindow(void){
  Gtk::Window *window = manage(new Gtk::Window());
  window->set_modal(true);
  window->set_title("Help");
  window->set_border_width(5);
  window->set_policy(0,0,0);
  window->set_usize(300,200);
  Gtk::Frame *helpframe=manage(new Gtk::Frame("Help"));
  Gtk::Label *helplabel=manage(new Gtk::Label("If you need help with this application\nthen tough luck :)\nIf you find any bugs please report them to me at \n<igge_rask@yahoo.com>\nor visit #trabzonspor on QuakeNET\n\ncheers"));
  helplabel->set_line_wrap(true);
  helpframe->add(*helplabel);
  window->add(*helpframe);
  window->show_all();
};

void MixerGUI::callback(int dev){
  int volume= (int) (100-adj[dev]->get_value());
  myMixer.SetLevel(dev,volume);
  mutebuttons[dev]->set_active(false);
};

void MixerGUI::mute(int dev){
  if (mutebuttons[dev]->get_active())
    myMixer.SetLevel(dev,0);
  else
    callback(dev);
};

void MixerGUI::SaveDefault(void){
  if (myMixer.SaveSettings()<0)
    statusbar->set("Error saving default settings...");
  else statusbar->set("Default settings saved...");
};

gint MixerGUI::UpdateLevels(void){
  LineInfo **infos=myMixer.getInfos();
  for (int i=0;i<myMixer.NumDevices();i++)
    if (infos[i]->active && !mutebuttons[i]->get_active()){
      adj[i]->set_value(100-myMixer.GetLevel(i));
    }
  return true;
};

void MixerGUI::LoadDefaults(void){
  if (myMixer.LoadSettings()<0)
    statusbar->set("Error loading default settings...");
  else statusbar->set("Default settings loaded...");
  UpdateLevels();
};

void MixerGUI::Load(void){
  fsel = manage(new Gtk::FileSelection("Load settings..."));
  if (strlen(lastload)>0 && lastload!=NULL) fsel->set_filename(lastload);
  fsel->show();
  fsel->get_cancel_button()->clicked.connect(fsel->destroy.slot());
  fsel->get_ok_button()->clicked.connect(slot(this, &MixerGUI::LoadCallback));
};

void MixerGUI::Save(void){
  fsel = manage(new Gtk::FileSelection("Save settings..."));
  if (lastsave!=NULL && strlen(lastsave)>0) fsel->set_filename(lastsave);
  fsel->show();
  fsel->get_cancel_button()->clicked.connect(fsel->destroy.slot());
  fsel->get_ok_button()->clicked.connect(slot(this, &MixerGUI::SaveCallback));
}

void MixerGUI::SaveCallback(void){
  if (myMixer.SaveSettings(fsel->get_filename().c_str())<0) {
    statusbar->set("Error saving settings to " + fsel->get_filename() + "...");
    fsel->destroy();
  } else {
    statusbar->set("Settings saved  to " + fsel->get_filename() + "...");
    delete lastsave;
    lastsave=new char[fsel->get_filename().length()+1];
    strcpy(lastsave,fsel->get_filename().c_str());
    UpdateLevels();
    fsel->destroy();
  }
};

void MixerGUI::LoadCallback(void){
  if (myMixer.LoadSettings(fsel->get_filename().c_str())<0){
    statusbar->set("Error loading settings from " + fsel->get_filename() + "...");
    fsel->destroy();
  }
  else {
    statusbar->set("Settings loaded from " + fsel->get_filename() + "...");
    delete lastload;
    lastload=new char[fsel->get_filename().length()+1];
    strcpy(lastload,fsel->get_filename().c_str());
    UpdateLevels();
    fsel->destroy();
  }
};

MixerGUI::MixerGUI (char *device) : 
  Gtk::Window(GTK_WINDOW_TOPLEVEL),
       myMixer(device),
       mainbox(false, 10),
       channelframe("Channels")
{
  using namespace Gtk;
  using namespace Menu_Helpers;
  scaletable = manage(new Table(3,myMixer.NumDevices(),false));
  menubar = new MenuBar();
  statusbar = manage(new Label(title));
  scaleboxes= new (VScale *)[myMixer.NumDevices()];
  adj = new (Adjustment *)[myMixer.NumDevices()];
  mutebuttons = new (CheckButton *)[myMixer.NumDevices()];
  //--- initialize some vars to avoid segfaults =P //
  set_policy(0,0,0);
  lastload=new char[3];
  lastsave=new char[3];
  title=new char[strlen(VERSION)+strlen("opMixer")];
  sprintf(title,"opMixer %s", VERSION);
  strcpy(lastload,"");
  strcpy(lastsave,"");
  //---- Main window design ------//
  set_title(title);
  destroy.connect(slot(&destroy_handler)); 
  set_border_width(10);
  scaletable->set_border_width(10);
  add(mainbox);
  mainbox.pack_start(*menubar,false);
  mainbox.pack_start(channelframe);
  statusbar->set_usize(300,15);

  Gtk::Main::timeout.connect(slot(this,&MixerGUI::TimerCallback),6000);
  Gtk::Main::timeout.connect(slot(this,&MixerGUI::UpdateLevels),100);
  // Add some handy timeouts :)

  mainbox.pack_start(*statusbar);
  channelframe.add(*scaletable);
  LineInfo **infos=myMixer.getInfos();
  int dev=0;
  Gtk::Alignment *align;
  for (int i=0;i<myMixer.NumDevices();i++){
    if (infos[i]->active) {
      adj[i]=new Adjustment(100-myMixer.GetLevel(i),0,100);
      mutebuttons[i]=manage(new CheckButton());
      mutebuttons[i]->set_usize(15,15);
      if (myMixer.GetLevel(i)==0)
	mutebuttons[i]->set_active(true);
      scaleboxes[i]=manage(new VScale(*adj[i]));
      scaleboxes[i]->set_usize(40,120);
      scaleboxes[i]->set_draw_value(false);
      adj[i]->value_changed.connect(bind<int>(slot(this,&MixerGUI::callback),i));
      mutebuttons[i]->clicked.connect(bind<int>(slot(this,&MixerGUI::mute),i));
      scaletable->attach(*scaleboxes[i],dev,dev+1,1,2);
      scaletable->attach(*manage(new Label(infos[i]->label)),dev,dev+1,0,1);
      align=manage(new Alignment(0.68,0,0.2,1));
      align->add(*mutebuttons[i]);
      scaletable->attach(*align,dev,dev+1,2,3);
      dev++;
    }
  };

  filemenu=manage(new Menu());
  helpmenu=manage(new Menu());
  MenuList& list_file=filemenu->items();
  list_file.push_back(MenuElem("_Load Defaults",'l',slot(this,&MixerGUI::LoadDefaults)));
  list_file.push_back(MenuElem("_Save Defaults",'s',slot(this,&MixerGUI::SaveDefault)));
  list_file.push_back(MenuElem("Load from file",slot(this,&MixerGUI::Load)));
  list_file.push_back(MenuElem("Save to file",slot(this,&MixerGUI::Save)));
  list_file.push_back(MenuElem("_Quit",'q',slot(&destroy_handler)));
  MenuList& list_bar = menubar->items();
  list_bar.push_front(MenuElem("_File","<alt>F",*filemenu));
  MenuList& list_help = helpmenu->items();
  list_help.push_back(MenuElem("_About opMixer",'A',slot(this,&MixerGUI::AboutWindow)));
  list_help.push_back(MenuElem("_Help",'H',slot(this,&MixerGUI::HelpWindow)));
  list_bar.push_back(MenuElem("_Help","<alt>H",*helpmenu));
  show_all();
};

int main(int argc, char *argv[])
{
  Gtk::Main Main(argc, argv);
  char *device;
  if (argc==2)
    device = argv[1];
  else 
    device = "/dev/mixer";
  MixerGUI g(device);
  Main.run();
  return(0);
}
