#include "ossmixer.hh"
#include <string.h>
#include <iostream.h>

ossMixer::ossMixer(char *dev){
  mixer_device=new char[strlen(dev)];
  strcpy(mixer_device,dev);
  init();
};

ossMixer::ossMixer(void){
  mixer_device="/dev/mixer";
  init();
};

void ossMixer::init(void){
  char *names[]=SOUND_DEVICE_NAMES;
  lineinfos = new (LineInfo *)[SOUND_MIXER_NRDEVICES];
  if (-1== (fd = open(mixer_device,O_RDWR))) {
    std::cout << endl
	      << "Error opening mixer device [" << mixer_device << "]" << endl
	      << "Please make sure permissions are set correctly." << endl
	      << endl;
    exit(EXIT_FAILURE);
  };
  int devmask,volume;
  ioctl(fd,SOUND_MIXER_READ_DEVMASK,&devmask);

  for (int i=0;i<SOUND_MIXER_NRDEVICES;i++){
    lineinfos[i]=new LineInfo;
    lineinfos[i]->label = new char[strlen(names[i])];
    strcpy(lineinfos[i]->label,names[i]);
    lineinfos[i]->active=false;
    lineinfos[i]->active= ((1<<i)&devmask);
    ioctl(fd,MIXER_READ(i),&volume);
    lineinfos[i]->volume=volume&0x00ff;
  };
};

ossMixer::~ossMixer(void){
  close(fd);
};

int ossMixer::NumDevices(void){
  return SOUND_MIXER_NRDEVICES;
};

int ossMixer::GetLevel(int dev) {
  int volume;
  ioctl(fd,MIXER_READ(dev),&volume);
  return (int) (volume&0x00ff);
};

void ossMixer::SetLevel(int dev, int volume){
  int vol=(volume<<8)|volume;
  ioctl(fd,MIXER_WRITE(dev),&vol);
};

void ossMixer::SetLevel(char *dev, int volume){
  for (int i=0;i<SOUND_MIXER_NRDEVICES;i++)
    if (!strcmp(lineinfos[i]->label,dev))
      SetLevel(i,volume);
};

LineInfo **ossMixer::getInfos(void){
  return lineinfos;
};

int ossMixer::SaveSettings(void){
  char *filename=new char[strlen(getenv("HOME"))+strlen("./opmixer")];
  strcpy(filename,getenv("HOME"));
  strcat(filename,"/.opmixer");
  return (SaveToFile(filename));
};

int ossMixer::SaveSettings(const char *filename){
  return (SaveToFile(filename));
};


int ossMixer::LoadSettings(void){
  char *filename=new char[strlen(getenv("HOME"))+strlen("./opmixer")];
  strcpy(filename,getenv("HOME"));
  strcat(filename,"/.opmixer");
  return (LoadFromFile(filename));
};

int ossMixer::LoadSettings(const char *filename){
  return (LoadFromFile(filename));
};

int ossMixer::SaveToFile(const char *filename){
  FILE *fp=fopen(filename,"w");
  if (fp==NULL) return -1;
  cout << "Saved config to " << filename << endl;
  fprintf(fp,"#Config file for opMixer, strings ARE case sensitive\n");
  for (int i=0;i<SOUND_MIXER_NRDEVICES;i++){
    if (lineinfos[i]->active) {
      fprintf(fp, "Label : %s\n", lineinfos[i]->label);
      fprintf(fp, "Level : %i\n\n", GetLevel(i));
    };
  };
  fclose(fp);
  return 0;
};

int ossMixer::LoadFromFile(const char *filename){
  FILE *fp=fopen(filename,"r");
  if (fp==NULL) return -1;
  cout << "Loading config from " << filename << endl;
  char *buffer=new char[4096];
  char *tlabel=new char[128];
  char *tlevel=new char[128];
  int level;
  fgets(buffer,256,fp);
  if (!strstr(buffer,"#Config file")) {
    fclose(fp);
    return -1;
  }
  while (fgets(buffer,4096,fp)){
    if (strstr(buffer,"Label : ")) {
      sscanf(buffer,"Label : %s\n", tlabel);
      fgets(buffer,4096,fp);
      sscanf(buffer,"Level : %s\n", tlevel);
      level=atoi(tlevel);
      SetLevel(tlabel,level);
    }
  };
  return 0;
};
